import re
import pandas as pd
from typing import List
from .config import (
    FDA_FILEPATH,
    USAN_FILEPATH,
    RXNORM_FILEPATH,
    SWISSMEDIC_FILEPATH,
)

def extract_drug_name(text):
    split_char = ','
    compound_raw = text.split(split_char)[0]
    pattern = r' [0-9]*'
    compound_wo_alone_numbers = re.sub(pattern, ' ', compound_raw)
    pattern2 = r'/[0-9]*'
    compound_number_filtered = re.sub(pattern2, ' ', compound_wo_alone_numbers)

    stopwords = ['a', 'b', 'c', 'g', 'n', 'p', 's', 'u', 'e.', 'ie', '%', 'i.e.',
                 '.0', '.05', '.1', '.2', '.3', '.4', '.5', '.6', '.625', '.7', '.8', '.9', '.25', '.75',
                 '9', '10', '15', '20',
                 '25', '30', '35', '43', '50', '52', '160', '200', '240', '300', '320', '400', '800', '.1%', '.25%',
                 '.3%', '.5%', '10%', '15%', '20%', "'000", "'250",
                 '.5mg', 'mcg', 'mg', 'ΜG', 'ml', 'mmol', 'l', 'ca', 'xr', 'ug', 'hg', 'cu', 'qu', 'arg.', 'c.',
                 'mikrogramm', 'liquid',
                 'gefärbt', 'ungefärbt', 'agenti', 'conservanti', 'nr.', 'adultes', 'enfants', 'zum',
                 'einnehmen', 'für', 'erwachsene', 'und', 'kinder', 'ab', 'jahren', 'äusserlich', 'con',
                 'spezifiziert', 'spécifié', 'extract', 'preparation', 'tabletten', 'pulver', 'zur',
                 'herstellung', 'einnehmen',
                 'sandoz', 'mundipharma', 'spirig', 'hc', 'maddox', 'bayer', 'nobel', 'zentiva',
                 'accord', 'labatec', 'ideogen', 'viatris', 'fresenius', 'pfizer', 'coop', 'vitality',
                 'sun', 'store', 'axapharm', 'orpha'
                 ]

    words_within_medicament = compound_number_filtered.split()
    result_words = [word for word in words_within_medicament if word.lower() not in stopwords]
    compound_filtered = ' '.join(result_words)

    if compound_filtered[0] == '-':
        compound_filtered = compound_filtered[1:]

    return compound_filtered


def read_medicament_file_as_list(chosen_sources: List):
    """
    read_medicament_file reads files from any of the four sources ['fda','rxnorm','usan','swissmedic']

    Input:
    chosen_sources: List with any number out of the possible sources ['fda','rxnorm','usan','swissmedic']

    Return:
    File: read individual file for that specific source.
    """
    if chosen_sources == 'fda':
        # Read file for comparison
        filepath = FDA_FILEPATH
        data = pd.read_csv(filepath, header=None, delimiter="|")
        name = pd.Series(data.iloc[:, [1, 4]].to_numpy().flatten())
        name = name.drop_duplicates()
        name = name.dropna().to_list()

    elif chosen_sources == 'usan':
        filepath = USAN_FILEPATH
        data = pd.read_csv(filepath, header='infer', delimiter=";")
        name = data.Examples.to_numpy().flatten()
        name = pd.Series(name).drop_duplicates()
        name = name.dropna()
        name = name.to_list()
        print("Number of drugs USAN database: ", len(name))

    elif chosen_sources == 'rxnorm':
        filepath = RXNORM_FILEPATH
        data = pd.read_csv(filepath, header=None, delimiter="|")
        name = data.iloc[:, 1].to_numpy().flatten()
        name = pd.Series(name).drop_duplicates()
        name = name.dropna().to_numpy()
        list_output = [drug.upper() for drug in name]
        print("Number of drugs at the RxNorm database: ", len(list_output))
        return list_output

    elif chosen_sources == 'swissmedic':
        filepath = SWISSMEDIC_FILEPATH
        data = pd.read_excel(filepath, skiprows=6, header=0)
        col_compounds = 'Bezeichnung des Arzneimittels\n\n\nDénomination du médicament'

        data['drug_name'] = data[col_compounds].apply(extract_drug_name)
        name = data['drug_name'].to_numpy().flatten()
        name = pd.Series(name).drop_duplicates()
        name = name.dropna().to_numpy()
        list_output = [drug.upper() for drug in name]
        print("Number of drugs at the Swissmedic database: ", len(list_output))
        return list_output

    else:
        name = False

    return name


import gradio as gr

# Sample JSON data for three example drugs with valid image URLs
iron_magic_milk_data = [
    {
        "offerRoot": "GOOGLE",
        "url": "https://www.sport-enzinger.com/kategorie/ernaehrung/aminosaeuren/komplexe-aminos/",
        "title": "/ komplexe Aminos",
        "price": "12.50 \u20ac",
        "zyteExecutionTime": 3.734952926635742,
        "zyteProbability": 0.005122258793562651,
        "fullDescription": "Amazing product sold here",
        "images": [
            "https://cdn.basler-beauty.de/out/pictures/generated/manufacturer/icon/100_100_100/milk-shake.b1107237.png",
        ],
        "pageType": "other",
    },
    {
        "offerRoot": "GOOGLE",
        "url": "https://www.labelhair.at/milkshake/sos-roots-mahogany",
        "price": "16.59 \u20ac",
        "title": "SOS Roots MAHOGANY, 75 ml",
        "fullDescription": "SOS Roots MAHAGONY von milk_shake ist ein nat\u00fcrliches Spray, das den Ansatz bis zu der n\u00e4chsten Haarw\u00e4sche in sattem Mahagoni kaschiert. Der feine, schnell trocknende Spr\u00fchnebel erm\u00f6glicht, dass das Haar sauber und trocken bleibt, ohne es zu beschweren. Die nat\u00fcrlichen Pigmente basieren auf Mineralien und sogen f\u00fcr eine intensivere Farbe und eine optimale Grauabdeckung.",
        "zyteExecutionTime": 2.3590247631073,
        "zyteProbability": 0.9918913841247559,
        "images": [
            "https://la.nice-cdn.com/upload/image/product/large/default/13228_d1afc42f.1024x1024.jpg"
        ],
        "pageType": "ecommerce_product",
    },
    {
        "offerRoot": "GOOGLE",
        "url": "https://www.basler-beauty.at/marken/milk-shake/milk-shake-light-catcher-fast-toning.html",
        "price": "5.9 \u20ac",
        "title": "milk_shake Light Catcher fast toning T\u00f6nung",
        "fullDescription": "5 NEUE FARBT\u00d6NE F\u00dcR TRAUMHAFT BLONDES HAAR. Alle Farbt\u00f6ne sind untereinander mischbar!\n\nEntdecken Sie die neuen milk_shake \u00ae light catcher fast toning Direktfarben, die das Light-Catcher-Sortiment bereichern und vervollst\u00e4ndigen. Empfohlen f\u00fcr Stufe 9 oder helleres blondes und augehelltes Haar.\n\n5 Farbt\u00f6ne f\u00fcr bereits aufgehelltes Haar:\n\nSofortige und zarte Farbt\u00f6ne\nDirektpigmente, die 4\u20135 Haarw\u00e4schen* halten\npflegt und stellt den nat\u00fcrlichen pH-Wert des Haares wieder her\nMit hochwertigen Pigmenten frei von Ammoniak und Oxidationsmitteln, Parabenen (SLS und SLES)\nAuffrischung der Haarfarbe zwischen den T\u00f6nungsbehandlungen\nErzeugen zarter Pastelleffekte, Verst\u00e4rken und Korrigieren von Farbt\u00f6nen\n\n*je nach Haarzustand, Einwirkzeit und Menge des verwendeten Produkts\n\nAnwendung: Gleichm\u00e4\u00dfig im sauberen, feuchten Haar verteilen oder je nach gew\u00fcnschtem Ergebnis auf ausgew\u00e4hlte Str\u00e4hnen auftragen. Nach der Einwirkzeit aufsch\u00e4umen und aussp\u00fclen. Falls erforderlich, mit einem Conditioner nachbehandeln. Einwirkzeit: zwischen 5 und 20 Minuten. Die Farbt\u00f6ne k\u00f6nnen einem Shampoo, einer Maske oder einer Pflegesp\u00fclung beigef\u00fcgt werden, um einen sanften Toning-Effekt zu erzielen.",
        "zyteExecutionTime": 3.5089142322540283,
        "zyteProbability": 0.7695555090904236,
        "images": [
            "https://cdn.basler-beauty.de/out/pictures/generated/product/1/980_980_100/40285fef8d320087018d63f40edc5f83-milk-shake-Light-Catcher-fast-toning.68603879.jpg",
            "https://cdn.basler-beauty.de/out/pictures/generated/manufacturer/icon/100_100_100/milk-shake.b1107237.png",
            "https://cdn.basler-beauty.de/out/pictures/generated/product/1/1200_1200_100/40285fef8d320087018d63f40edc5f83-milk-shake-Light-Catcher-fast-toning.68603879.jpg",
        ],
        "pageType": "ecommerce_product",
    },
    {
        "offerRoot": "GOOGLE",
        "url": "https://marienapotheke.at/en/produkt/oleovital-eisen-classic-12mg/?add-to-cart=34814",
        "price": "28.6 \u20ac",
        "title": "Oleovital Iron Classic 12 mg sachets",
        "fullDescription": "Iron contributes to normal cognitive function.\n\nIron supports the formation of red blood cells and hemoglobin.\n\nIron supports oxygen transport and folic acid, vitamin B6, vitamin B12 and vitamin C reduce symptoms of tiredness.\n\nIron supports the energy metabolism.\n\nContents: 30 sachets with cola flavor",
        "zyteExecutionTime": 3.840689182281494,
        "zyteProbability": 0.9885560870170593,
        "images": [
            "https://marienapotheke.at/wp-content/uploads/2024/08/OLEOvital_Eisen_classic_ergebnis.webp"
        ],
        "pageType": "ecommerce_product",
    },
]

devalife_data = [
    {
        "offerRoot": "GOOGLE",
        "url": "https://www.medizinfuchs.at/devalife-kapseln.html",
        "title": "Alle Angebote zu Devalife Kapseln",
        "fullDescription": "26 Angebote zu Devalife kapseln im Medikamenten Preisvergleich. Devalife kapseln g\u00fcnstig kaufen und sparen bei medizinfuchs.at",
        "zyteExecutionTime": 3.1472229957580566,
        "zyteProbability": 0.005644466727972031,
        "images": ["https://www.medizinfuchs.at/images/medizinfuchs_logo_big.png"],
        "price": "16.59 \u20ac",
        "pageType": "other",
    },
    {
        "offerRoot": "EBAY",
        "url": "https://www.ebay.at/itm/166811727812",
        "price": "60.0 EUR",
        "title": "DEVALIFE 1 Packungen 30 Kapsel 1 Monate Kur",
        "fullDescription": "Der Verk\u00e4ufer ist f\u00fcr dieses Angebot verantwortlich.\n\neBay-Artikelnr.:166811727812\n\nArtikelmerkmale\n\nArtikelzustand\nNeu: Neuer, unbenutzter und unbesch\u00e4digter Artikel in nicht ge\u00f6ffneter Originalverpackung (soweit ... Mehr erfahren\u00dcber den ZustandNeu: Neuer, unbenutzter und unbesch\u00e4digter Artikel in nicht ge\u00f6ffneter Originalverpackung (soweit eine Verpackung vorhanden ist). Die Verpackung sollte der im Einzelhandel entsprechen. Ausnahme: Der Artikel war urspr\u00fcnglich in einer Nichteinzelhandelsverpackung verpackt, z. B. unbedruckter Karton oder Plastikh\u00fclle. Weitere Einzelheiten im Angebot des Verk\u00e4ufers. Alle Zustandsdefinitionen ansehenwird in neuem Fenster oder Tab ge\u00f6ffnet\n\nHerstellernummer\nnicht zutreffend\n\nMarke\nDevalife\n\nFormulierung\nKapseln\n\nHerstellungsland und -region\nT\u00fcrkei\n\nAngebotspaket\nJa\n\nMindesthaltbarkeitsdatum\n2025\n\nWirksame Inhaltsstoffe\nHimbeere, Ballaststoff, Gr\u00fcntee-Extrakt, Guarana, Fenchel, Koriander, Heidekrautbl\u00e4tter, Rosmarin\n\nArtikelbeschreibung des Verk\u00e4ufers",
        "zyteExecutionTime": 2.830410957336426,
        "zyteProbability": 0.9628998041152954,
        "images": [
            "https://i.ebayimg.com/images/g/VbwAAOSwy1NaZQPn/s-l500.webp",
            "https://i.ebayimg.com/images/g/sHQAAOSwk-hkDaYx/s-l500.webp",
            "https://i.ebayimg.com/images/g/WlYAAOSw0txkDaYy/s-l140.webp",
            "https://i.ebayimg.com/images/g/VbwAAOSwy1NaZQPn/s-l140.webp",
            "https://i.ebayimg.com/images/g/ayAAAOSwjVhkDaYz/s-l500.webp",
            "https://i.ebayimg.com/images/g/ayAAAOSwjVhkDaYz/s-l1600.webp",
            "https://i.ebayimg.com/images/g/UcUAAOSwoRBaZQVk/s-l140.webp",
            "https://i.ebayimg.com/images/g/Rn0AAOSwIk9aZQU0/s-l140.webp",
            "https://i.ebayimg.com/images/g/WlYAAOSw0txkDaYy/s-l500.webp",
            "https://i.ebayimg.com/images/g/HD8AAOSwZFJkDaXo/s-l140.webp",
            "https://i.ebayimg.com/images/g/ayAAAOSwjVhkDaYz/s-l960.webp",
            "https://i.ebayimg.com/images/g/HD8AAOSwZFJkDaXo/s-l960.webp",
            "https://i.ebayimg.com/images/g/Rn0AAOSwIk9aZQU0/s-l960.webp",
            "https://i.ebayimg.com/images/g/ayAAAOSwjVhkDaYz/s-l140.webp",
            "https://i.ebayimg.com/images/g/WlYAAOSw0txkDaYy/s-l960.webp",
            "https://i.ebayimg.com/images/g/VbwAAOSwy1NaZQPn/s-l960.webp",
            "https://i.ebayimg.com/images/g/KJUAAOSwoFtkDaY~/s-l140.webp",
            "https://i.ebayimg.com/images/g/UcUAAOSwoRBaZQVk/s-l500.webp",
            "https://i.ebayimg.com/images/g/sHQAAOSwk-hkDaYx/s-l140.webp",
            "https://i.ebayimg.com/images/g/Zp8AAOSwEMpkDaX1/s-l500.webp",
            "https://i.ebayimg.com/images/g/KJUAAOSwoFtkDaY~/s-l500.webp",
            "https://i.ebayimg.com/images/g/Zp8AAOSwEMpkDaX1/s-l140.webp",
            "https://i.ebayimg.com/images/g/Rn0AAOSwIk9aZQU0/s-l1600.webp",
            "https://i.ebayimg.com/images/g/KJUAAOSwoFtkDaY~/s-l1600.webp",
            "https://i.ebayimg.com/images/g/sHQAAOSwk-hkDaYx/s-l1600.webp",
            "https://i.ebayimg.com/images/g/Zp8AAOSwEMpkDaX1/s-l1600.webp",
            "https://i.ebayimg.com/images/g/UcUAAOSwoRBaZQVk/s-l960.webp",
            "https://i.ebayimg.com/thumbs/images/g/KJUAAOSwoFtkDaY~/s-l500.jpg",
            "https://i.ebayimg.com/images/g/UcUAAOSwoRBaZQVk/s-l1600.webp",
            "https://i.ebayimg.com/images/g/VbwAAOSwy1NaZQPn/s-l1600.webp",
            "https://i.ebayimg.com/images/g/HD8AAOSwZFJkDaXo/s-l500.webp",
            "https://i.ebayimg.com/images/g/sHQAAOSwk-hkDaYx/s-l960.webp",
            "https://i.ebayimg.com/images/g/Rn0AAOSwIk9aZQU0/s-l500.webp",
            "https://i.ebayimg.com/images/g/WlYAAOSw0txkDaYy/s-l1600.webp",
            "https://i.ebayimg.com/images/g/Zp8AAOSwEMpkDaX1/s-l960.webp",
            "https://i.ebayimg.com/images/g/HD8AAOSwZFJkDaXo/s-l1600.webp",
        ],
        "pageType": "ecommerce_product",
    },
]


# Function to process the input and display the JSON data
def display_results(site_token, serp_token, country, search_term, selected_example):
    # Check if any of the custom configuration fields are filled
    if site_token or serp_token or search_term:
        return "<h3>Reach out to us if you want to know more about how to use this sandbox environment</h3>"

    # Select the example data based on the dropdown choice
    if selected_example == "Devalife":
        data = devalife_data
    elif selected_example == "Iron Magic Milk":
        data = iron_magic_milk_data
    else:
        return "Please select a valid example."

    # Generate HTML content
    html_content = ""
    for item in data:
        image_url = item["images"][0] if item["images"] else ""
        html_content += f"""
        <div style='border:1px solid #ccc; padding:10px; margin-bottom:10px;'>
            <h2>{item['title']}</h2>
            <img src='{image_url}' alt='Image' style='max-width:200px;'/>
            <p><strong>Price:</strong> {item['price']}</p>
            <p><strong>Description:</strong> {item['fullDescription']}</p>
            <p><a href='{item['url']}'>View Product</a></p>
        </div>
        """
    return html_content


# Define the Gradio app layout
with gr.Blocks(title="Nightcrawler Sandbox (Vianus' Demo)") as app:
    gr.Markdown(
        """
        <h1 style="text-align: center;">Nightcrawler Sandbox (Vianus' Demo)</h1>
        """,
        elem_id="centered-title",
    )
    with gr.Row():
        with gr.Column(scale=1):
            # First Box: Examples
            gr.Markdown("### Examples")
            example_dropdown = gr.Dropdown(
                label="Select Example Drug",
                choices=[
                    "Devalife",
                    "Iron Magic Milk",
                ],
                value="Devalife",
                interactive=True,
            )
            country_dropdown = gr.Dropdown(
                label="Country",
                choices=["CH", "CL", "AT", "ALL"],
                value="AT",
                interactive=True,
            )

            # Second Box: Custom Configuration (collapsible)
            gr.Markdown("### Custom Configuration")
            with gr.Accordion("Search for your own keywords", open=False):
                zyte_api = gr.Textbox(
                    label="Zyte API (Name Token)",
                    type="password",
                    placeholder="Enter Zyte API Token",
                )
                serp_api = gr.Textbox(
                    label="Serp API (Name Token)",
                    type="password",
                    placeholder="Enter Serp API Token",
                )
                search_term = gr.Textbox(
                    label="Search Term", placeholder="Enter your search term"
                )

        # Results section
        with gr.Column(scale=2):
            results = gr.HTML(label="Results")

    # Set up the function to run when inputs are filled
    display_button = gr.Button("Display Results")
    display_button.click(
        fn=display_results,
        inputs=[zyte_api, serp_api, country_dropdown, search_term, example_dropdown],
        outputs=results,
    )


def main():
    app.launch(debug=True, share=True)


# Launch the app
if __name__ == "__main__":
    main()



import logging

# Create a named logger
logger = logging.getLogger("nightcrawler_logger")

# Set the log level
logger.setLevel(logging.INFO)

# Define a custom formatter
formatter = logging.Formatter(
    fmt="%(asctime)s | %(name)s | %(levelname)s | %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
)

# Create a console handler
console_handler = logging.StreamHandler()
console_handler.setFormatter(formatter)

# Add the handler to the logger
logger.addHandler(console_handler)



from pathlib import Path

ROOT_PATH = Path(__file__).parents[2]
STATIC_PATH = ROOT_PATH / 'datasets'
OUTPUT_PATH = ROOT_PATH / 'output'

FDA_FILEPATH = STATIC_PATH / 'drugsatfda_20210527.csv'
USAN_FILEPATH = STATIC_PATH / '20211125_USAN_stem_list_cumulative_csv_2.csv'
RXNORM_FILEPATH = STATIC_PATH / 'drugs_rxnorm_20210510.csv'
SWISSMEDIC_FILEPATH = STATIC_PATH / 'swissmedic_zugelassene_arzneimittel_20230531_public.xlsx'


import pandas as pd
from .load_files import read_medicament_file_as_list
from .matching_sequences import match_seq_against_list
from pandas import DataFrame

def collapse_sources(df_drugs_identified):
    """
    Function joins sources when a medicament is identified multiple times

    :return: dataframe with deduplicated rows
    """
    # Concatenate string
    df_drugs_identified['dataset'] = df_drugs_identified.groupby(
        ['name'])[['dataset']].transform(lambda x: ', '.join(x))
    # drop duplicate data
    df_drugs_identified = df_drugs_identified.drop_duplicates()
    return df_drugs_identified


def verify_dataframe_has_items(results: DataFrame):
    if len(results) > 0:
        return results
    else:
        # Create the new row as a DataFrame
        new_row = pd.DataFrame([{
            'name': 'No object found',
            'gram': 0,
            'phon': 0,
            'phon-fr-1': 0,
            'phon-fr-2': 0,
            'dataset': 'none',
            'comb': 0
        }])

        results = pd.concat([results, new_row], ignore_index=True)

        return results

def search(searched_word: str, sources, threshold: float = 50) -> pd.DataFrame:
    """
    Main Search Function

    """

    # UPPERCASE SEARCHED WORD, WHERE SEARCHED_WORD IS STRING SUBMITTED BY USER.
    searched_word = str(searched_word).upper()

    # INITIALIZE 'data_matched' VARIABLE. LIST WITH MATCHES THAT IS LATER READ AS DATAFRAME.
    data_matched = []
    for source_name in sources:
        read_list = read_medicament_file_as_list(source_name)
        data_matched = match_seq_against_list(data_matched, read_list, source_name, searched_word, float(threshold))

    res = pd.DataFrame(data_matched, columns=['name', 'gram', 'phon-de', 'phon-fr', 'dataset'])

    # COMBINATION SCORE GERMAN
    res['comb'] = res[['gram', 'phon-de']].mean(axis=1)
    res = res.sort_values(by='comb', ascending=False)

    # COMBINATION SCORE FRENCH
    res['comb-fr'] = res[['gram', 'phon-fr']].mean(axis=1)

    # VERIFY IT IS EMPTY
    print(res)
    res = verify_dataframe_has_items(res)
    print(res)

    # COLLAPSE SOURCES AND DEDUPLICATE
    res = collapse_sources(res)

    # res.to_csv(OUTPUT_PATH / 'result_' + searched_word + '.csv', header=None)
    return res


import os

from dotenv import load_dotenv
from vianu.tools.nightcrawler.src.client import NightcrawlerClient

load_dotenv()

# Instantiate the client
nc_client = NightcrawlerClient()

# Set API tokens
nc_client.serpapi_token = os.getenv("SERP_API_TOKEN", "YOUR_SERPAPI_TOKEN")
nc_client.zyte_api_key = os.getenv("ZYTE_API_TOKEN", "YOUR_ZYTE_API_KEY")

# Perform search
df = nc_client.search("sildenafil", num_results=5, location="Switzerland")

# Display results
df


import requests
import logging

logger = logging.getLogger("nightcrawler_logger")

class SerpApiClient:
    """
    A client to interact with the SERP API for performing search queries.
    """

    def __init__(self, serpapi_token, location):
        """
        Initializes the SerpApiClient with the given API token.

        Args:
            serpapi_token (str): The API token for SERP API.
        """
        self.serpapi_token = serpapi_token
        self.location = location

    def search(self, query, num_results=10):
        """
        Performs a search using SERP API and returns the URLs of the results.

        Args:
            query (str): The search query.
            num_results (int): Number of results to return.

        Returns:
            list: A list of URLs from the search results.
        """
        logger.info(f"Performing SERP API search for query: {query}")

        params = {
            "engine": "google",
            "q": query,
            "api_key": self.serpapi_token,
            "num": num_results,
            "location_requested": self.location,
            "location_used": self.location,
        }

        response = requests.get("https://serpapi.com/search", params=params)

        if response.status_code == 200:
            data = response.json()
            search_results = data.get("organic_results", [])
            urls = [result.get("link") for result in search_results]
            logger.info(f"Found {len(urls)} URLs from SERP API.")
            return urls
        else:
            logger.error(
                f"SERP API request failed with status code {response.status_code}"
            )
            return []


from typing import List
from .comparison_utils import orthographic_comparison, phonetic_comparison


def match_seq_against_list(data_matched: List, database: List, name_source: str, searched_word, threshold: float):
    """
    Function goes through each database comparing word of interest "searched_word" with every element and collects only
    those that stay above a given threshold.
    """
    for each in database:
        ratio = orthographic_comparison(searched_word, each, 'SequenceMatcher')

        ratio_phonetic = phonetic_comparison(searched_word, each, 'Kolner')

        ratio_fr_phonetic_fonem = phonetic_comparison(searched_word, each, 'fr-fonem')

        # THIS FUNCTION ADDED AN ALTERNATIVE FRENCH PHONETIC COMPARISON - DISCONTINUED IN MARCH 2022.
        # ratio_fr_phonetic_henry = phonetic_comparison(searched_word, each, 'fr-henry')

        ratio_combined = (ratio + ratio_phonetic) / 2
        if ratio_combined >= (float(threshold)/100):
            data_matched.append([each,
                                 round(ratio * 100),
                                 round(ratio_phonetic * 100),
                                 round(ratio_fr_phonetic_fonem * 100),
                                 name_source])

    return data_matched


import logging

logger = logging.getLogger("nightcrawler_logger")


class Processor:
    """
    Processes the product data and applies specific filtering rules.
    """

    def __init__(self, country_code):
        """
        Initializes the Processor with the given country code.

        Args:
            country_code (str): The country code to filter results by.
        """
        self.country_code = country_code.lower()

    def process(self, products):
        """
        Processes the product data and filters based on country code.

        Args:
            products (list): A list of product data dictionaries.

        Returns:
            list: A filtered list of product data dictionaries.
        """
        logger.info(
            f"Processing {len(products)} products and filtering by country code: {self.country_code.upper()}"
        )

        filtered_products = []
        for product in products:
            url = product.get("url", "")
            if (
                f".{self.country_code}/" in url.lower()
                or url.lower().endswith(f".{self.country_code}")
                or ".com" in url.lower()
            ):
                filtered_products.append(product)

        logger.info(
            f"Filtered down to {len(filtered_products)} products after applying country code filter."
        )
        return filtered_products




import pandas as pd
import logging

from vianu.tools.nightcrawler.src.processor import Processor
from vianu.tools.nightcrawler.src.serpapi import SerpApiClient
from vianu.tools.nightcrawler.src.zyteapi import ZyteApiClient

logger = logging.getLogger("nightcrawler_logger")


class NightcrawlerClient:
    """
    The main client that orchestrates the search, data fetching, and processing.
    """

    def __init__(self, serpapi_token=None, zyte_api_key=None):
        """
        Initializes the NightcrawlerClient with optional API tokens.

        Args:
            serpapi_token (str, optional): The API token for SERP API.
            zyte_api_key (str, optional): The API key for Zyte API.
        """
        self.serpapi_token = serpapi_token
        self.zyte_api_key = zyte_api_key

    def search(self, query, location, num_results=10, country_code="ch"):
        """
        Performs the search, gets product details, processes them, and returns a DataFrame.

        Args:
            query (str): The search query.
            num_results (int): Number of search results to process.
            country_code (str): The country code to filter results.

        Returns:
            DataFrame: A pandas DataFrame containing the final product data.
        """
        # Ensure API tokens are set
        if not self.serpapi_token:
            raise ValueError("SERP API token is not set.")
        if not self.zyte_api_key:
            raise ValueError("Zyte API key is not set.")

        # Instantiate clients
        serp_client = SerpApiClient(self.serpapi_token, location)
        zyte_client = ZyteApiClient(self.zyte_api_key)
        processor = Processor(country_code)

        # Perform search
        urls = serp_client.search(query, num_results)
        if not urls:
            logger.error("No URLs found from SERP API.")
            return pd.DataFrame()

        # Get product details
        products = zyte_client.get_product_details(urls)
        if not products:
            logger.error("No product details fetched from Zyte API.")
            return pd.DataFrame()

        # Process products
        filtered_products = processor.process(products)
        if not filtered_products:
            logger.warning("No products left after filtering.")
            return pd.DataFrame()

        # Flatten the product data
        df = pd.json_normalize(filtered_products)

        # Log and return the DataFrame
        logger.info("Search completed. Returning flattened DataFrame.")
        return df




import requests
import logging
import time
from tqdm.auto import tqdm
from requests.auth import HTTPBasicAuth

logger = logging.getLogger("nightcrawler_logger")


class ZyteApiClient:
    """
    A client to interact with the Zyte API for fetching product details.
    """

    def __init__(self, zyte_api_key, max_retries=1, retry_delay=10):
        """
        Initializes the ZyteApiClient with the given API key and retry configurations.

        Args:
            zyte_api_key (str): The API key for Zyte API.
            max_retries (int): Maximum number of retries for API calls.
            retry_delay (int): Delay between retries in seconds.
        """
        self.endpoint = "https://api.zyte.com/v1/extract"
        self.auth = HTTPBasicAuth(zyte_api_key, "")
        self.max_retries = max_retries
        self.retry_delay = retry_delay

    def get_product_details(self, urls):
        """
        Fetches product details from the given URLs using Zyte API.

        Args:
            urls (list): A list of URLs to fetch product details from.

        Returns:
            list: A list of dictionaries containing product details.
        """
        logger.info(f"Fetching product details for {len(urls)} URLs via Zyte API.")
        products = []
        config = {
            "javascript": False,
            "browserHtml": False,
            "screenshot": False,
            "product": True,
            "productOptions": {"extractFrom": "httpResponseBody"},
            "httpResponseBody": True,
            "geolocation": "CH",
            "viewport": {"width": 1280, "height": 1080},
            "actions": [],
        }

        with tqdm(total=len(urls)) as pbar:
            for url in urls:
                attempts = 0
                while attempts < self.max_retries:
                    try:
                        logger.debug(
                            f"Attempting to fetch product details for URL: {url} (Attempt {attempts + 1})"
                        )
                        response = requests.post(
                            self.endpoint,
                            auth=self.auth,
                            json={
                                "url": url,
                                **config,
                            },
                            timeout=10,
                        )

                        if response.status_code == 200:
                            product_data = response.json()
                            product_data["url"] = url  # Ensure the URL is included
                            products.append(product_data)
                            logger.debug(
                                f"Successfully fetched product details for URL: {url}"
                            )
                            break  # Exit the retry loop on success
                        else:
                            logger.error(
                                f"Zyte API request failed for URL {url} with status code {response.status_code} "
                                f"and response: {response.text}"
                            )
                            attempts += 1
                            if attempts < self.max_retries:
                                logger.warning(
                                    f"Retrying in {self.retry_delay} seconds..."
                                )
                                time.sleep(self.retry_delay)
                    except Exception as e:
                        logger.error(
                            f"Exception occurred while fetching product details for URL {url}: {e}"
                        )
                        attempts += 1
                        if attempts < self.max_retries:
                            logger.warning(f"Retrying in {self.retry_delay} seconds...")
                            time.sleep(self.retry_delay)
                else:
                    logger.error(f"All attempts failed for URL: {url}")
                pbar.update(1)

        logger.info(f"Fetched product details for {len(products)} URLs.")
        return products




import importlib
from typing import List
from difflib import SequenceMatcher
from abydos.phonetic import fonem, henry_early
cologne = importlib.import_module("cologne_phonetics")


def orthographic_comparison(searched_word: str, element_list: List, option: str) -> float:
    """
    This function takes a "searched_string" which is the name that we interested in and compares it
    against the "element_list" which are the names that are pulled from the large datasets. Short said,
    we are interested for example in the name "MAVIX", and compare again the entire list one by one

    :param searched_word: medicament name that we are interested in
    :param element_list: medicament names from the databases
    :param option: given phonetic technic for comparison
    :return: phonetic_score: which is the estimation of the phonetic matching
    """
    if option == 'SequenceMatcher':
        ratio = SequenceMatcher(None, searched_word, element_list).ratio()
    return ratio


def phonetic_comparison(searched_word, element_list, option):
    """
    drug_testname = searched_string
    drug_found = element_list

    This function takes a "searched_string" which is the name that we interested in and compares it
    against the "element_list" which are the names that are pulled from the large datasets. Short said,
    we are interested for example in the name "MAVIX", and compare again the entire list one by one

    :param searched_word: medicament name that we are interested in
    :param element_list: medicament names from the databases
    :param option: given phonetic technic for comparison
    :return: phonetic_score: which is the estimation of the phonetic matching
    """
    ratio = -1
    if option == 'Kolner':
        drug_interest = cologne.encode(searched_word.upper())[0][1]
        drug_in_our_list = cologne.encode(element_list.upper())[0][1]
        ratio = SequenceMatcher(None, drug_interest, drug_in_our_list).ratio()

        phonetic_score = float(round(ratio, 2))

    if option == 'fr-fonem':
        drug_interest = fonem(searched_word.upper())
        drug_in_our_list = fonem(element_list.upper())
        ratio = SequenceMatcher(None, drug_interest, drug_in_our_list).ratio()
        phonetic_score = float(round(ratio, 2))

    if option == 'fr-henry':
        drug_interest = henry_early(searched_word.upper())
        drug_in_our_list = henry_early(element_list.upper())
        ratio = SequenceMatcher(None, drug_interest, drug_in_our_list).ratio()
        phonetic_score = float(round(ratio, 2))

    return phonetic_score